@@ -26,14 +26,19 @@ def pipeline():
 
 @pipeline.command()
 @click.argument("path", type=click.Path(exists=True), required=False, default=".")
-def upload(path):
+@click.option(
+    '--no-lockfile',
+    is_flag=True,
+    help='Skip creating config-lock.yaml file.',
+)
+def upload(path, no_lockfile):
     """Upload a pipeline with associated pipeline steps to Clarifai.
 
     PATH: Path to the pipeline configuration file or directory containing config.yaml. If not specified, the current directory is used by default.
     """
     from clarifai.runners.pipelines.pipeline_builder import upload_pipeline
 
-    upload_pipeline(path)
+    upload_pipeline(path, no_lockfile=no_lockfile)
 
 
 @pipeline.command()
@@ -106,15 +111,32 @@ def run(
 
     validate_context(ctx)
 
+    # Try to load from config-lock.yaml first if no config is specified
+    lockfile_path = os.path.join(os.getcwd(), "config-lock.yaml")
+    if not config and os.path.exists(lockfile_path):
+        logger.info("Found config-lock.yaml, using it as default config source")
+        config = lockfile_path
+
     if config:
         config_data = from_yaml(config)
-        pipeline_id = config_data.get('pipeline_id', pipeline_id)
-        pipeline_version_id = config_data.get('pipeline_version_id', pipeline_version_id)
+
+        # Handle both regular config format and lockfile format
+        if 'pipeline' in config_data and isinstance(config_data['pipeline'], dict):
+            pipeline_config = config_data['pipeline']
+            pipeline_id = pipeline_config.get('id', pipeline_id)
+            pipeline_version_id = pipeline_config.get('version_id', pipeline_version_id)
+            user_id = pipeline_config.get('user_id', user_id)
+            app_id = pipeline_config.get('app_id', app_id)
+        else:
+            # Fallback to flat config structure
+            pipeline_id = config_data.get('pipeline_id', pipeline_id)
+            pipeline_version_id = config_data.get('pipeline_version_id', pipeline_version_id)
+            user_id = config_data.get('user_id', user_id)
+            app_id = config_data.get('app_id', app_id)
+
         pipeline_version_run_id = config_data.get(
             'pipeline_version_run_id', pipeline_version_run_id
         )
-        user_id = config_data.get('user_id', user_id)
-        app_id = config_data.get('app_id', app_id)
         nodepool_id = config_data.get('nodepool_id', nodepool_id)
         compute_cluster_id = config_data.get('compute_cluster_id', compute_cluster_id)
         pipeline_url = config_data.get('pipeline_url', pipeline_url)
@@ -229,12 +251,34 @@ def init(pipeline_path):
     # Create the pipeline directory if it doesn't exist
     os.makedirs(pipeline_path, exist_ok=True)
 
+    # Prompt for user inputs
+    click.echo("Welcome to Clarifai Pipeline Initialization!")
+    click.echo("Please provide the following information:")
+
+    user_id = click.prompt("User ID", type=str)
+    app_id = click.prompt("App ID", type=str)
+    pipeline_id = click.prompt("Pipeline ID", default="hello-world-pipeline", type=str)
+    num_steps = click.prompt("Number of pipeline steps", default=2, type=int)
+
+    # Get step names
+    step_names = []
+    default_names = ["stepA", "stepB", "stepC", "stepD", "stepE", "stepF"]
+
+    for i in range(num_steps):
+        default_name = default_names[i] if i < len(default_names) else f"step{i + 1}"
+        step_name = click.prompt(f"Name for step {i + 1}", default=default_name, type=str)
+        step_names.append(step_name)
+
+    click.echo(f"\nCreating pipeline '{pipeline_id}' with steps: {', '.join(step_names)}")
+
     # Create pipeline config.yaml
     config_path = os.path.join(pipeline_path, "config.yaml")
     if os.path.exists(config_path):
         logger.warning(f"File {config_path} already exists, skipping...")
     else:
-        config_template = get_pipeline_config_template()
+        config_template = get_pipeline_config_template(
+            pipeline_id=pipeline_id, user_id=user_id, app_id=app_id, step_names=step_names
+        )
         with open(config_path, 'w', encoding='utf-8') as f:
             f.write(config_template)
         logger.info(f"Created {config_path}")
@@ -249,8 +293,8 @@ def init(pipeline_path):
             f.write(readme_template)
         logger.info(f"Created {readme_path}")
 
-    # Create pipeline steps (stepA and stepB)
-    for step_id in ["stepA", "stepB"]:
+    # Create pipeline steps
+    for step_id in step_names:
         step_dir = os.path.join(pipeline_path, step_id)
         os.makedirs(step_dir, exist_ok=True)
 
@@ -263,7 +307,9 @@ def init(pipeline_path):
         if os.path.exists(step_config_path):
             logger.warning(f"File {step_config_path} already exists, skipping...")
         else:
-            step_config_template = get_pipeline_step_config_template(step_id)
+            step_config_template = get_pipeline_step_config_template(
+                step_id=step_id, user_id=user_id, app_id=app_id
+            )
             with open(step_config_path, 'w', encoding='utf-8') as f:
                 f.write(step_config_template)
             logger.info(f"Created {step_config_path}")
@@ -290,13 +336,65 @@ def init(pipeline_path):
 
     logger.info(f"Pipeline initialization complete in {pipeline_path}")
     logger.info("Next steps:")
-    logger.info("1. Search for '# TODO: please fill in' comments in the generated files")
-    logger.info("2. Update your user_id and app_id in all config.yaml files")
-    logger.info(
-        "3. Implement your pipeline step logic in stepA/1/pipeline_step.py and stepB/1/pipeline_step.py"
-    )
-    logger.info("4. Add dependencies to requirements.txt files as needed")
-    logger.info("5. Run 'clarifai pipeline upload config.yaml' to upload your pipeline")
+    logger.info("1. Implement your pipeline step logic in the generated pipeline_step.py files")
+    logger.info("2. Add dependencies to requirements.txt files as needed")
+    logger.info("3. Run 'clarifai pipeline upload config.yaml' to upload your pipeline")
+
+
+@pipeline.command()
+@click.argument(
+    "lockfile_path", type=click.Path(exists=True), required=False, default="config-lock.yaml"
+)
+def validate_lock(lockfile_path):
+    """Validate a config-lock.yaml file for schema and reference consistency.
+
+    LOCKFILE_PATH: Path to the config-lock.yaml file. If not specified, looks for config-lock.yaml in current directory.
+    """
+    from clarifai.runners.utils.pipeline_validation import PipelineConfigValidator
+    from clarifai.utils.cli import from_yaml
+
+    try:
+        # Load the lockfile
+        lockfile_data = from_yaml(lockfile_path)
+
+        # Validate required fields
+        if "pipeline" not in lockfile_data:
+            raise ValueError("'pipeline' section not found in lockfile")
+
+        pipeline = lockfile_data["pipeline"]
+        required_fields = ["id", "user_id", "app_id", "version_id"]
+
+        for field in required_fields:
+            if field not in pipeline:
+                raise ValueError(f"Required field '{field}' not found in pipeline section")
+            if not pipeline[field]:
+                raise ValueError(f"Required field '{field}' cannot be empty")
+
+        # Validate orchestration spec if present
+        if "orchestration_spec" in pipeline:
+            # Create a temporary config structure for validation
+            temp_config = {
+                "pipeline": {
+                    "id": pipeline["id"],
+                    "user_id": pipeline["user_id"],
+                    "app_id": pipeline["app_id"],
+                    "orchestration_spec": pipeline["orchestration_spec"],
+                }
+            }
+
+            # Use existing validator to check orchestration spec
+            validator = PipelineConfigValidator()
+            validator._validate_orchestration_spec(temp_config)
+
+        logger.info(f"✅ Lockfile {lockfile_path} is valid")
+        logger.info(f"Pipeline: {pipeline['id']}")
+        logger.info(f"User: {pipeline['user_id']}")
+        logger.info(f"App: {pipeline['app_id']}")
+        logger.info(f"Version: {pipeline['version_id']}")
+
+    except Exception as e:
+        logger.error(f"❌ Lockfile validation failed: {e}")
+        raise click.Abort()
 
 
 @pipeline.command(['ls'])
