@@ -1,6 +1,6 @@
 import time
 import uuid
-from typing import Dict, List
+from typing import Dict, List, Optional
 
 from clarifai_grpc.grpc.api import resources_pb2, service_pb2
 from clarifai_grpc.grpc.api.status import status_code_pb2
@@ -19,35 +19,35 @@ class Pipeline(Lister, BaseClient):
 
     def __init__(
         self,
-        url: str = None,
-        pipeline_id: str = None,
-        pipeline_version_id: str = None,
-        pipeline_version_run_id: str = None,
-        user_id: str = None,
-        app_id: str = None,
-        nodepool_id: str = None,
-        compute_cluster_id: str = None,
-        log_file: str = None,
+        url: Optional[str] = None,
+        pipeline_id: Optional[str] = None,
+        pipeline_version_id: Optional[str] = None,
+        pipeline_version_run_id: Optional[str] = None,
+        user_id: Optional[str] = None,
+        app_id: Optional[str] = None,
+        nodepool_id: Optional[str] = None,
+        compute_cluster_id: Optional[str] = None,
+        log_file: Optional[str] = None,
         base_url: str = DEFAULT_BASE,
-        pat: str = None,
-        token: str = None,
-        root_certificates_path: str = None,
+        pat: Optional[str] = None,
+        token: Optional[str] = None,
+        root_certificates_path: Optional[str] = None,
         **kwargs,
     ):
         """Initializes a Pipeline object.
 
         Args:
-            url (str): The URL to initialize the pipeline object.
-            pipeline_id (str): The Pipeline ID to interact with.
-            pipeline_version_id (str): The Pipeline Version ID to interact with.
-            pipeline_version_run_id (str): The Pipeline Version Run ID. If not provided, a UUID will be generated.
-            user_id (str): The User ID that owns the pipeline.
-            app_id (str): The App ID that contains the pipeline.
-            nodepool_id (str): The Nodepool ID to run the pipeline on.
-            compute_cluster_id (str): The Compute Cluster ID to run the pipeline on.
-            log_file (str): Path to file where logs should be written. If not provided, logs are displayed on console.
+            url (Optional[str]): The URL to initialize the pipeline object.
+            pipeline_id (Optional[str]): The Pipeline ID to interact with.
+            pipeline_version_id (Optional[str]): The Pipeline Version ID to interact with.
+            pipeline_version_run_id (Optional[str]): The Pipeline Version Run ID. If not provided, a UUID will be generated.
+            user_id (Optional[str]): The User ID that owns the pipeline.
+            app_id (Optional[str]): The App ID that contains the pipeline.
+            nodepool_id (Optional[str]): The Nodepool ID to run the pipeline on.
+            compute_cluster_id (Optional[str]): The Compute Cluster ID to run the pipeline on.
+            log_file (Optional[str]): Path to file where logs should be written. If not provided, logs are displayed on console.
             base_url (str): Base API url. Default "https://api.clarifai.com"
-            pat (str): A personal access token for authentication. Can be set as env var CLARIFAI_PAT
+            pat (Optional[str]): A personal access token for authentication. Can be set as env var CLARIFAI_PAT
             token (str): A session token for authentication. Accepts either a session token or a pat. Can be set as env var CLARIFAI_SESSION_TOKEN
             root_certificates_path (str): Path to the SSL root certificates file, used to establish secure gRPC connections.
             **kwargs: Additional keyword arguments to be passed to the Pipeline.
@@ -193,6 +193,7 @@ class Pipeline(Lister, BaseClient):
         """
         start_time = time.time()
         seen_logs = set()
+        current_page = 1  # Track current page for log pagination.
 
         while time.time() - start_time < timeout:
             # Get run status
@@ -217,8 +218,8 @@ class Pipeline(Lister, BaseClient):
                     pipeline_run, preserving_proto_field_name=True
                 )
 
-                # Display new log entries
-                self._display_new_logs(run_id, seen_logs)
+                # Display new log entries and update current page
+                current_page = self._display_new_logs(run_id, seen_logs, current_page)
 
                 elapsed_time = time.time() - start_time
                 logger.info(f"Pipeline run monitoring... (elapsed {elapsed_time:.1f}s)")
@@ -276,12 +277,16 @@ class Pipeline(Lister, BaseClient):
         logger.error(f"Pipeline run timed out after {timeout} seconds")
         return {"status": "timeout"}
 
-    def _display_new_logs(self, run_id: str, seen_logs: set):
+    def _display_new_logs(self, run_id: str, seen_logs: set, current_page: int = 1) -> int:
         """Display new log entries for a pipeline version run.
 
         Args:
             run_id (str): The pipeline version run ID.
             seen_logs (set): Set of already seen log entry IDs.
+            current_page (int): The current page to fetch logs from.
+
+        Returns:
+            int: The next page number to fetch from in subsequent calls.
         """
         try:
             logs_request = service_pb2.ListLogEntriesRequest()
@@ -290,7 +295,7 @@ class Pipeline(Lister, BaseClient):
             logs_request.pipeline_version_id = self.pipeline_version_id or ""
             logs_request.pipeline_version_run_id = run_id
             logs_request.log_type = "pipeline.version.run"  # Set required log type
-            logs_request.page = 1
+            logs_request.page = current_page
             logs_request.per_page = 50
 
             logs_response = self.STUB.ListLogEntries(
@@ -298,7 +303,9 @@ class Pipeline(Lister, BaseClient):
             )
 
             if logs_response.status.code == status_code_pb2.StatusCode.SUCCESS:
+                entries_count = 0
                 for log_entry in logs_response.log_entries:
+                    entries_count += 1
                     # Use log entry URL or timestamp as unique identifier
                     log_id = log_entry.url or f"{log_entry.created_at.seconds}_{log_entry.message}"
                     if log_id not in seen_logs:
@@ -312,5 +319,14 @@ class Pipeline(Lister, BaseClient):
                         else:
                             logger.info(log_message)
 
+                # If we got a full page (50 entries), there might be more logs on the next page
+                # If we got fewer than 50 entries, we've reached the end and should stay on current page
+                if entries_count == 50:
+                    return current_page + 1
+                else:
+                    return current_page
+
         except Exception as e:
             logger.debug(f"Error fetching logs: {e}")
+            # Return current page on error to retry the same page next fetch
+            return current_page
