@@ -1,20 +1,21 @@
 import time
-from typing import Iterator
+from typing import Iterator, Optional, Union
 
 from clarifai_grpc.grpc.api import service_pb2
 from clarifai_grpc.grpc.api.status import status_code_pb2, status_pb2
 from clarifai_protocol import BaseRunner
-from clarifai_protocol.utils.health import HealthProbeRequestHandler
+from clarifai_protocol.utils.health import HealthProbeRequestHandler, start_health_server_thread
 
 from clarifai.client.auth.helper import ClarifaiAuthHelper
 from clarifai.utils.constants import STATUS_FAIL, STATUS_MIXED, STATUS_OK, STATUS_UNKNOWN
 from clarifai.utils.logging import get_req_id_from_context, logger
+from clarifai.utils.secrets import inject_secrets, req_secrets_context
 
 from ..utils.url_fetcher import ensure_urls_downloaded
 from .model_class import ModelClass
 
 
-class ModelRunner(BaseRunner, HealthProbeRequestHandler):
+class ModelRunner(BaseRunner):
     """
     This is a subclass of the runner class which will handle only the work items relevant to models.
     """
@@ -25,12 +26,13 @@ class ModelRunner(BaseRunner, HealthProbeRequestHandler):
         runner_id: str,
         nodepool_id: str,
         compute_cluster_id: str,
-        user_id: str = None,
+        user_id: Optional[str] = None,
         check_runner_exists: bool = True,
         base_url: str = "https://api.clarifai.com",
-        pat: str = None,
-        token: str = None,
+        pat: Optional[str] = None,
+        token: Optional[str] = None,
         num_parallel_polls: int = 4,
+        health_check_port: Union[int, None] = 8080,
         **kwargs,
     ) -> None:
         super().__init__(
@@ -54,7 +56,7 @@ class ModelRunner(BaseRunner, HealthProbeRequestHandler):
         self._base_url = base_url
 
         # Create auth helper if we have sufficient authentication information
-        self._auth_helper = None
+        self._auth_helper: Optional[ClarifaiAuthHelper] = None
         if self._user_id and (self._pat or self._token):
             try:
                 self._auth_helper = ClarifaiAuthHelper(
@@ -69,10 +71,27 @@ class ModelRunner(BaseRunner, HealthProbeRequestHandler):
                 # If auth helper creation fails, proceed without authentication
                 self._auth_helper = None
 
+        # if the model has a handle_liveness_probe method, call it to determine liveness
+        # otherwise rely on self.is_alive from the protocol
+        if hasattr(self.model, 'handle_liveness_probe'):
+            HealthProbeRequestHandler.handle_liveness_probe = self.model.handle_liveness_probe
+
+        # if the model has a handle_readiness_probe method, call it to determine readiness
+        # otherwise rely on self.is_ready from the protocol
+        if hasattr(self.model, 'handle_readiness_probe'):
+            HealthProbeRequestHandler.handle_readiness_probe = self.model.handle_readiness_probe
+
+        # if the model has a handle_startup_probe method, call it to determine startup
+        # otherwise rely on self.is_startup from the protocol
+        if hasattr(self.model, 'handle_startup_probe'):
+            HealthProbeRequestHandler.handle_startup_probe = self.model.handle_startup_probe
+
         # After model load successfully set the health probe to ready and startup
         HealthProbeRequestHandler.is_ready = True
         HealthProbeRequestHandler.is_startup = True
 
+        start_health_server_thread(port=health_check_port, address='')
+
     def get_runner_item_output_for_status(
         self, status: status_pb2.Status
     ) -> service_pb2.RunnerItemOutput:
@@ -109,6 +128,7 @@ class ModelRunner(BaseRunner, HealthProbeRequestHandler):
             raise Exception("Unexpected work item type: {}".format(runner_item))
         request = runner_item.post_model_outputs_request
         ensure_urls_downloaded(request, auth_helper=self._auth_helper)
+        inject_secrets(request)
         start_time = time.time()
         req_id = get_req_id_from_context()
         status_str = STATUS_UNKNOWN
@@ -124,7 +144,10 @@ class ModelRunner(BaseRunner, HealthProbeRequestHandler):
         if method_name == '_GET_SIGNATURES':
             logging = False
 
-        resp = self.model.predict_wrapper(request)
+        # Use req_secrets_context to temporarily set request-type secrets as environment variables
+        with req_secrets_context(request):
+            resp = self.model.predict_wrapper(request)
+
         # if we have any non-successful code already it's an error we can return.
         if (
             resp.status.code != status_code_pb2.SUCCESS
@@ -175,6 +198,7 @@ class ModelRunner(BaseRunner, HealthProbeRequestHandler):
             raise Exception("Unexpected work item type: {}".format(runner_item))
         request = runner_item.post_model_outputs_request
         ensure_urls_downloaded(request, auth_helper=self._auth_helper)
+        inject_secrets(request)
 
         # --- Live logging additions ---
         start_time = time.time()
@@ -182,45 +206,49 @@ class ModelRunner(BaseRunner, HealthProbeRequestHandler):
         status_str = STATUS_UNKNOWN
         endpoint = "POST /v2/.../outputs/generate"
 
-        for resp in self.model.generate_wrapper(request):
-            # if we have any non-successful code already it's an error we can return.
-            if (
-                resp.status.code != status_code_pb2.SUCCESS
-                and resp.status.code != status_code_pb2.ZERO
-            ):
-                status_str = f"{resp.status.code} ERROR"
-                duration_ms = (time.time() - start_time) * 1000
-                logger.info(f"{endpoint} | {status_str} | {duration_ms:.2f}ms | req_id={req_id}")
-                yield service_pb2.RunnerItemOutput(multi_output_response=resp)
-                continue
-            successes = []
-            for output in resp.outputs:
-                if not output.HasField('status') or not output.status.code:
-                    raise Exception(
-                        "Output must have a status code, please check the model implementation."
+        # Use req_secrets_context to temporarily set request-type secrets as environment variables
+        with req_secrets_context(request):
+            for resp in self.model.generate_wrapper(request):
+                # if we have any non-successful code already it's an error we can return.
+                if (
+                    resp.status.code != status_code_pb2.SUCCESS
+                    and resp.status.code != status_code_pb2.ZERO
+                ):
+                    status_str = f"{resp.status.code} ERROR"
+                    duration_ms = (time.time() - start_time) * 1000
+                    logger.info(
+                        f"{endpoint} | {status_str} | {duration_ms:.2f}ms | req_id={req_id}"
                     )
-                successes.append(output.status.code == status_code_pb2.SUCCESS)
-            if all(successes):
-                status = status_pb2.Status(
-                    code=status_code_pb2.SUCCESS,
-                    description="Success",
-                )
-                status_str = STATUS_OK
-            elif any(successes):
-                status = status_pb2.Status(
-                    code=status_code_pb2.MIXED_STATUS,
-                    description="Mixed Status",
-                )
-                status_str = STATUS_MIXED
-            else:
-                status = status_pb2.Status(
-                    code=status_code_pb2.FAILURE,
-                    description="Failed",
-                )
-                status_str = STATUS_FAIL
-            resp.status.CopyFrom(status)
+                    yield service_pb2.RunnerItemOutput(multi_output_response=resp)
+                    continue
+                successes = []
+                for output in resp.outputs:
+                    if not output.HasField('status') or not output.status.code:
+                        raise Exception(
+                            "Output must have a status code, please check the model implementation."
+                        )
+                    successes.append(output.status.code == status_code_pb2.SUCCESS)
+                if all(successes):
+                    status = status_pb2.Status(
+                        code=status_code_pb2.SUCCESS,
+                        description="Success",
+                    )
+                    status_str = STATUS_OK
+                elif any(successes):
+                    status = status_pb2.Status(
+                        code=status_code_pb2.MIXED_STATUS,
+                        description="Mixed Status",
+                    )
+                    status_str = STATUS_MIXED
+                else:
+                    status = status_pb2.Status(
+                        code=status_code_pb2.FAILURE,
+                        description="Failed",
+                    )
+                    status_str = STATUS_FAIL
+                resp.status.CopyFrom(status)
 
-            yield service_pb2.RunnerItemOutput(multi_output_response=resp)
+                yield service_pb2.RunnerItemOutput(multi_output_response=resp)
 
         duration_ms = (time.time() - start_time) * 1000
         logger.info(f"{endpoint} | {status_str} | {duration_ms:.2f}ms | req_id={req_id}")
@@ -234,53 +262,68 @@ class ModelRunner(BaseRunner, HealthProbeRequestHandler):
         status_str = STATUS_UNKNOWN
         endpoint = "POST /v2/.../outputs/stream  "
 
-        for resp in self.model.stream_wrapper(pmo_iterator(runner_item_iterator)):
-            # if we have any non-successful code already it's an error we can return.
-            if (
-                resp.status.code != status_code_pb2.SUCCESS
-                and resp.status.code != status_code_pb2.ZERO
-            ):
-                status_str = f"{resp.status.code} ERROR"
-                duration_ms = (time.time() - start_time) * 1000
-                logger.info(f"{endpoint} | {status_str} | {duration_ms:.2f}ms | req_id={req_id}")
-                yield service_pb2.RunnerItemOutput(multi_output_response=resp)
-                continue
-            successes = []
-            for output in resp.outputs:
-                if not output.HasField('status') or not output.status.code:
-                    raise Exception(
-                        "Output must have a status code, please check the model implementation."
-                    )
-                successes.append(output.status.code == status_code_pb2.SUCCESS)
-            if all(successes):
-                status = status_pb2.Status(
-                    code=status_code_pb2.SUCCESS,
-                    description="Success",
-                )
-                status_str = STATUS_OK
-            elif any(successes):
-                status = status_pb2.Status(
-                    code=status_code_pb2.MIXED_STATUS,
-                    description="Mixed Status",
-                )
-                status_str = STATUS_MIXED
-            else:
-                status = status_pb2.Status(
-                    code=status_code_pb2.FAILURE,
-                    description="Failed",
-                )
-                status_str = STATUS_FAIL
-            resp.status.CopyFrom(status)
+        # Get the first request to establish secrets context
+        first_request = None
+        runner_items = list(runner_item_iterator)  # Convert to list to avoid consuming iterator
+        if runner_items:
+            first_request = runner_items[0].post_model_outputs_request
 
-            yield service_pb2.RunnerItemOutput(multi_output_response=resp)
+        # Use req_secrets_context based on the first request (secrets should be consistent across stream)
+        with req_secrets_context(first_request):
+            for resp in self.model.stream_wrapper(pmo_iterator(iter(runner_items))):
+                # if we have any non-successful code already it's an error we can return.
+                if (
+                    resp.status.code != status_code_pb2.SUCCESS
+                    and resp.status.code != status_code_pb2.ZERO
+                ):
+                    status_str = f"{resp.status.code} ERROR"
+                    duration_ms = (time.time() - start_time) * 1000
+                    logger.info(
+                        f"{endpoint} | {status_str} | {duration_ms:.2f}ms | req_id={req_id}"
+                    )
+                    yield service_pb2.RunnerItemOutput(multi_output_response=resp)
+                    continue
+                successes = []
+                for output in resp.outputs:
+                    if not output.HasField('status') or not output.status.code:
+                        raise Exception(
+                            "Output must have a status code, please check the model implementation."
+                        )
+                    successes.append(output.status.code == status_code_pb2.SUCCESS)
+                if all(successes):
+                    status = status_pb2.Status(
+                        code=status_code_pb2.SUCCESS,
+                        description="Success",
+                    )
+                    status_str = STATUS_OK
+                elif any(successes):
+                    status = status_pb2.Status(
+                        code=status_code_pb2.MIXED_STATUS,
+                        description="Mixed Status",
+                    )
+                    status_str = STATUS_MIXED
+                else:
+                    status = status_pb2.Status(
+                        code=status_code_pb2.FAILURE,
+                        description="Failed",
+                    )
+                    status_str = STATUS_FAIL
+                resp.status.CopyFrom(status)
+
+                yield service_pb2.RunnerItemOutput(multi_output_response=resp)
 
         duration_ms = (time.time() - start_time) * 1000
         logger.info(f"{endpoint} | {status_str} | {duration_ms:.2f}ms | req_id={req_id}")
 
+    def set_model(self, model: ModelClass):
+        """Set the model for this runner."""
+        self.model = model
+
 
 def pmo_iterator(runner_item_iterator, auth_helper=None):
     for runner_item in runner_item_iterator:
         if not runner_item.HasField('post_model_outputs_request'):
             raise Exception("Unexpected work item type: {}".format(runner_item))
         ensure_urls_downloaded(runner_item.post_model_outputs_request, auth_helper=auth_helper)
+        inject_secrets(runner_item.post_model_outputs_request)
         yield runner_item.post_model_outputs_request
