@@ -33,6 +33,7 @@ def generate_client_script(
     compute_cluster_id: str = None,
     nodepool_id: str = None,
     use_ctx: bool = False,
+    colorize: bool = False,
 ) -> str:
     url_helper = ClarifaiUrlHelper()
 
@@ -91,7 +92,7 @@ response = client.chat.completions.create(
             "content": "How do I check if a Python object is an instance of a class?",
         }},
     ],
-    temperature=0.7,
+    temperature=1.0,
     stream=False,  # stream=True also works, just iterator over the response
 )
 print(response)
@@ -203,6 +204,16 @@ model = Model.from_current_context()
     script_lines.append(method_signatures_str)
     script_lines.append("")
     script = "\n".join(script_lines)
+    if colorize:
+        try:
+            from pygments import highlight  # type: ignore
+            from pygments.formatters import TerminalFormatter  # type: ignore
+            from pygments.lexers import PythonLexer  # type: ignore
+
+            return highlight(script, PythonLexer(), TerminalFormatter())
+        except Exception:
+            # Fallback to plain text if pygments is unavailable
+            return script
     return script
 
 
